@@ -13,7 +13,8 @@
 package Directory::Queue;
 use strict;
 use warnings;
-our $VERSION = sprintf("%d.%02d", q$Revision: 1.20 $ =~ /(\d+)\.(\d+)/);
+our $VERSION  = "1.1_1";
+our $REVISION = sprintf("%d.%02d", q$Revision: 1.30 $ =~ /(\d+)\.(\d+)/);
 
 #
 # used modules
@@ -41,10 +42,8 @@ use constant OBSOLETE_DIRECTORY => "obsolete";
 # name of the directory indicating a locked element
 use constant LOCKED_DIRECTORY => "locked";
 
-# states returned by the _state() method
-use constant STATE_UNLOCKED => "U";
-use constant STATE_LOCKED   => "L";
-use constant STATE_MISSING  => "M";
+# reasonable buffer size for file I/O operations
+use constant SYSBUFSIZE => 8192;
 
 #
 # global variables
@@ -53,12 +52,14 @@ use constant STATE_MISSING  => "M";
 our(
     $_DirectoryRegexp,    # regexp matching an intermediate directory
     $_ElementRegexp,      # regexp matching an element
+    $_FileRegexp,	  # regexp matching a file in an element directory
     %_Byte2Esc,           # byte to escape map
     %_Esc2Byte,           # escape to byte map
 );
 
 $_DirectoryRegexp = qr/[0-9a-f]{8}/;
 $_ElementRegexp   = qr/[0-9a-f]{14}/;
+$_FileRegexp      = qr/[0-9a-zA-Z]+/;
 
 %_Byte2Esc = ("\x5c" => "\\\\", "\x09" => "\\t", "\x0a" => "\\n");
 %_Esc2Byte = reverse(%_Byte2Esc);
@@ -88,7 +89,7 @@ sub _fatal ($@) {
 
 sub _file_read ($$) {
     my($path, $utf8) = @_;
-    my($fh, $contents, $done);
+    my($fh, $data, $done);
 
     sysopen($fh, $path, O_RDONLY)
 	or _fatal("cannot sysopen(%s, O_RDONLY): %s", $path, $!);
@@ -99,14 +100,14 @@ sub _file_read ($$) {
 	binmode($fh)
 	    or _fatal("cannot binmode(%s): %s", $path, $!);
     }
-    $contents = "";
+    $data = "";
     $done = -1;
     while ($done) {
-	$done = sysread($fh, $contents, 8192, length($contents));
+	$done = sysread($fh, $data, SYSBUFSIZE, length($data));
 	_fatal("cannot sysread(%s): %s", $path, $!) unless defined($done);
     }
     close($fh) or _fatal("cannot close(%s): %s", $path, $!);
-    return($contents);
+    return(\$data);
 }
 
 #
@@ -114,7 +115,7 @@ sub _file_read ($$) {
 #
 
 sub _file_write ($$$$) {
-    my($path, $utf8, $umask, $contents) = @_;
+    my($path, $utf8, $umask, $dataref) = @_;
     my($fh, $oldumask, $success, $length, $offset, $done);
 
     if (defined($umask)) {
@@ -124,7 +125,7 @@ sub _file_write ($$$$) {
     } else {
 	$success = sysopen($fh, $path, O_WRONLY|O_CREAT|O_EXCL);
     }
-    $success or _fatal("cannot sysopen(%s, O_WRONLY|O_CREAT|O_EXCL): %s", $path, $!);
+    _fatal("cannot sysopen(%s, O_WRONLY|O_CREAT|O_EXCL): %s", $path, $!) unless $success;
     if ($utf8) {
 	binmode($fh, ":encoding(utf8)")
 	    or _fatal("cannot binmode(%s, :encoding(utf8)): %s", $path, $!);
@@ -132,10 +133,10 @@ sub _file_write ($$$$) {
 	binmode($fh)
 	    or _fatal("cannot binmode(%s): %s", $path, $!);
     }
-    $length = length($contents);
+    $length = length($$dataref);
     $offset = 0;
     while ($length) {
-	$done = syswrite($fh, $contents, 8192, $offset);
+	$done = syswrite($fh, $$dataref, SYSBUFSIZE, $offset);
 	_fatal("cannot syswrite(%s): %s", $path, $!) unless defined($done);
 	$length -= $done;
 	$offset += $done;
@@ -144,7 +145,7 @@ sub _file_write ($$$$) {
 }
 
 #
-# transform a hash of strings into a string
+# transform a hash of strings into a string (reference)
 #
 # note:
 #  - the keys are sorted so that identical hashes yield to identical strings
@@ -163,21 +164,21 @@ sub _hash2string ($) {
 	$value =~ s/([\x5c\x09\x0a])/$_Byte2Esc{$1}/g;
 	$string .= $key . "\x09" . $value . "\x0a";
     }
-    return($string);
+    return(\$string);
 }
 
 #
-# transform a string into a hash of strings
+# transform a string (reference) into a hash of strings
 #
 # note:
 #  - duplicate keys are not checked (the last one wins)
 #
 
 sub _string2hash ($) {
-    my($string) = @_;
+    my($stringref) = @_;
     my($line, $key, $value, %hash);
 
-    foreach $line (split(/\x0a/, $string)) {
+    foreach $line (split(/\x0a/, $$stringref)) {
 	_fatal("unexpected hash line: %s", $line)
 	    unless $line =~ /^([^\x09\x0a]*)\x09([^\x09\x0a]*)$/o;
 	($key, $value) = ($1, $2);
@@ -239,13 +240,13 @@ sub _older ($$) {
 #  - return undef if the directory does not exist (anymore)
 #  - die in case of any other error
 #
-# note:
-#  - lstat() is used so symlinks are not followed
-#  - this only checks the number of links
-#  - we do not even check that the path indeed points to a directory!
-#
 
-sub _subdirs ($) {
+# stat version (faster):
+#  - lstat() is used so symlinks are not followed
+#  - this only checks the number of hard links
+#  - we do not even check that the given path indeed points to a directory!
+
+sub _subdirs_stat ($) {
     my($path) = @_;
     my(@stat);
 
@@ -255,11 +256,27 @@ sub _subdirs ($) {
 	# RACE: this path does not exist (anymore)
 	return();
     }
-    return($stat[ST_NLINK] - 2) unless $^O =~ /^(cygwin|dos|MSWin32)$/;
-    # argh! we cannot rely on the number of links on Windows :-(
+    return($stat[ST_NLINK] - 2);
+}
+
+# readdir version (slower):
+#  - we really count the number of entries
+#  - we however do not check that these entries are themselves indeed directories
+
+sub _subdirs_readdir ($) {
+    my($path) = @_;
+
     return(scalar(_directory_contents($path, 1)));
 }
 
+# use the right version (we cannot rely on hard links on DOS-like systems)
+
+if ($^O =~ /^(cygwin|dos|MSWin32)$/) {
+    *_subdirs = \&_subdirs_readdir;
+} else {
+    *_subdirs = \&_subdirs_stat;
+}
+
 #
 # create a directory:
 #  - return true on success
@@ -335,7 +352,7 @@ sub _check_element ($) {
     my($element) = @_;
 
     _fatal("invalid element: %s", $element)
-	unless $element =~ m/^($_DirectoryRegexp)\/($_ElementRegexp)$/o;
+	unless $element =~ m/^(?:$_DirectoryRegexp)\/(?:$_ElementRegexp)$/o;
 }
 
 #+++############################################################################
@@ -350,7 +367,7 @@ sub _check_element ($) {
 
 sub new : method {
     my($class, %option) = @_;
-    my($self, $name, $path, @stat);
+    my($self, $name, $options, $path, @stat);
 
     # default object
     $self = {
@@ -359,12 +376,12 @@ sub new : method {
 	maxelts => 16_000,    # maximum number of elements allowed per directory
     };
     # check options
-    _fatal("missing option: path") unless $option{path};
+    _fatal("missing option: path") unless defined($option{path});
     foreach $name (qw(path umask maxelts)) {
  	next unless defined($option{$name});
+	_fatal("invalid %s: %s", $name, $option{$name})
+	    if ref($option{$name});
 	$self->{$name} = delete($option{$name});
-	_fatal("invalid %s: %s", $name, $self->{$name})
-	    if ref($self->{$name});
     }
     # check umask
     if (defined($self->{umask})) {
@@ -377,16 +394,20 @@ sub new : method {
 	    unless $self->{maxelts} =~ /^\d+$/ and $self->{maxelts} > 0;
     }
     # check schema
-    if ($option{schema}) {
+    if (defined($option{schema})) {
 	_fatal("invalid schema: %s", $option{schema})
 	    unless ref($option{schema}) eq "HASH";
 	foreach $name (keys(%{ $option{schema} })) {
 	    _fatal("invalid schema name: %s", $name)
-		unless $name =~ /^\w+$/ and $name ne LOCKED_DIRECTORY;
+		unless $name =~ /^($_FileRegexp)$/ and $name ne LOCKED_DIRECTORY;
 	    _fatal("invalid schema type: %s", $option{schema}{$name})
-		unless $option{schema}{$name} =~ /^(binary|string|table)(\?)?$/;
+		unless $option{schema}{$name} =~ /^(binary|string|table)([\?\*]{0,2})$/;
 	    $self->{type}{$name} = $1;
-	    $self->{mandatory}{$name} = 1 unless $2;
+	    $options = $2;
+	    $self->{mandatory}{$name} = 1 unless $options =~ /\?/;
+	    $self->{ref}{$name} = 1 if $options =~ /\*/;
+	    _fatal("invalid schema type: %s", $option{schema}{$name})
+		if $self->{type}{$name} eq "table" and $self->{ref}{$name};
 	}
 	_fatal("invalid schema: no mandatory data")
 	    unless $self->{mandatory};
@@ -428,6 +449,7 @@ sub new : method {
 # note:
 #  - the main purpose is to copy/clone the iterator cached state
 #  - the other structured attributes (including schema) are not cloned
+#    (this is not a problem as they should not change)
 #
 
 sub copy : method {
@@ -461,24 +483,6 @@ sub id : method {
     return($self->{id});
 }
 
-#
-# return the state of the given element
-#
-# note:
-#  - this is only an indication as the state may be changed by an other process
-#
-
-sub _state : method {
-    my($self, $element) = @_;
-    my($path);
-
-    $path = $self->{path} . "/" . $element;
-    return(STATE_LOCKED)   if -d $path . "/" . LOCKED_DIRECTORY;
-    return(STATE_UNLOCKED) if -d $path;
-    # the element does not exist (anymore)
-    return(STATE_MISSING);
-}
-
 #
 # return the name of the next element in the queue, using cached information
 #
@@ -538,6 +542,59 @@ sub count : method {
     return($count);
 }
 
+#
+# check if an element is locked:
+#  - this is best effort only as it may change while we test (only locking is atomic)
+#  - if given a time, only return true on locks older than this time (needed by purge)
+#
+
+# version using nlink (faster)
+
+sub _is_locked_nlink : method {
+    my($self, $name, $time) = @_;
+    my($path, @stat);
+
+    $path = $self->{path} . "/" . $name;
+    @stat = lstat($path);
+    unless (@stat) {
+	_fatal("cannot lstat(%s): %s", $path, $!) unless $! == ENOENT;
+	# RACE: this path does not exist (anymore)
+	return(0);
+    }
+    # locking increases nlink so...
+    return(0) unless $stat[ST_NLINK] > 2;
+    # check age if time is given
+    return(0) if defined($time) and $stat[ST_MTIME] >= $time;
+    # so far so good but we double check that the proper directory does exist
+    return(-d $path . "/" . LOCKED_DIRECTORY);
+}
+
+# version not using nlink (slower)
+
+sub _is_locked_nonlink : method {
+    my($self, $name, $time) = @_;
+    my($path, @stat);
+
+    $path = $self->{path} . "/" . $name;
+    return(0) unless -d $path . "/" . LOCKED_DIRECTORY;
+    return(1) unless defined($time);
+    @stat = lstat($path);
+    unless (@stat) {
+	_fatal("cannot lstat(%s): %s", $path, $!) unless $! == ENOENT;
+	# RACE: this path does not exist (anymore)
+	return(0);
+    }
+    return($stat[ST_MTIME] < $time);
+}
+
+# use the right version (we cannot rely on hard links on DOS-like systems)
+
+if ($^O =~ /^(cygwin|dos|MSWin32)$/) {
+    *_is_locked = \&_is_locked_nonlink;
+} else {
+    *_is_locked = \&_is_locked_nlink;
+}
+
 #
 # lock an element:
 #  - return true on success
@@ -582,7 +639,8 @@ sub lock : method {
     unless (lstat($path)) {
 	if ($permissive) {
 	    # RACE: the element directory does not exist anymore
-	    # (this can happen if an other process locked & removed the element)
+	    # (this can happen if an other process locked & removed the element
+	    #  while our mkdir() was in progress... yes, this can happen!)
 	    return(0) if $! == ENOENT;
 	}
 	# otherwise this is unexpected...
@@ -633,21 +691,21 @@ sub remove : method {
     my($temp, $name, $path);
 
     _check_element($element);
-    _fatal("cannot remove %s: not locked", $element)
-	unless $self->_state($element) eq STATE_LOCKED;
+    _fatal("cannot remove %s: not locked", $element) unless $self->_is_locked($element);
     # move the element out of its intermediate directory
     $path = $self->{path} . "/" . $element;
     while (1) {
 	$temp = $self->{path} . "/" . OBSOLETE_DIRECTORY . "/" . _new_name();
 	rename($path, $temp) and last;
-	_fatal("cannot rename(%s, %s): %s", $path, $temp, $!) unless $! == ENOTEMPTY;
+	_fatal("cannot rename(%s, %s): %s", $path, $temp, $!)
+	    unless $! == ENOTEMPTY or $! == EEXIST;
 	# RACE: the target directory was already present...
     }
     # remove the data files
     foreach $name (_directory_contents($temp)) {
 	next if $name eq LOCKED_DIRECTORY;
 	_fatal("unexpected file in %s: %s", $temp, $name)
-	    unless $name =~ /^(\w+)$/;
+	    unless $name =~ /^($_FileRegexp)$/o;
 	$path = $temp . "/" . $1; # untaint
 	unlink($path) and next;
 	_fatal("cannot unlink(%s): %s", $path, $!);
@@ -657,9 +715,11 @@ sub remove : method {
     while (1) {
 	rmdir($path) or _fatal("cannot rmdir(%s): %s", $path, $!);
 	rmdir($temp) and return;
-	_fatal("cannot rmdir(%s): %s", $temp, $!) unless $! == ENOTEMPTY;
+	_fatal("cannot rmdir(%s): %s", $temp, $!)
+	    unless $! == ENOTEMPTY or $! == EEXIST;
 	# RACE: this can happen if an other process managed to lock this element
-	# while it was being removed so we try again to remove the lock
+	# while it was being removed (see the comment in the lock() method)
+	# so we try to remove the lock again and again...
     }
 }
 
@@ -669,12 +729,11 @@ sub remove : method {
 
 sub get : method {
     my($self, $element) = @_;
-    my(%data, $name, $path);
+    my(%data, $name, $path, $ref);
 
     _fatal("unknown schema") unless $self->{type};
     _check_element($element);
-    _fatal("cannot get %s: not locked", $element)
-	unless $self->_state($element) eq STATE_LOCKED;
+    _fatal("cannot get %s: not locked", $element) unless $self->_is_locked($element);
     foreach $name (keys(%{ $self->{type} })) {
 	$path = "$self->{path}/$element/$name";
 	unless (lstat($path)) {
@@ -685,16 +744,16 @@ sub get : method {
 		next;
 	    }
 	}
-	if ($self->{type}{$name} eq "binary") {
-	    $data{$name} = _file_read($path, 0);
-	} elsif ($self->{type}{$name} eq "string") {
-	    $data{$name} = _file_read($path, 1);
+	if ($self->{type}{$name} =~ /^(binary|string)$/) {
+	    $ref = _file_read($path, $self->{type}{$name} eq "string");
 	} elsif ($self->{type}{$name} eq "table") {
-	    $data{$name} = _string2hash(_file_read($path, 1));
+	    $ref = _string2hash(_file_read($path, 1));
 	} else {
 	    _fatal("unexpected data type: %s", $self->{type}{$name});
 	}
+	$data{$name} = $self->{ref}{$name} ? $ref : $$ref;
     }
+    return(\%data) unless wantarray();
     return(%data);
 }
 
@@ -749,40 +808,48 @@ sub _insertion_directory : method {
 #
 
 sub add : method {
-    my($self, %data) = @_;
-    my($temp, $dir, $name, $path);
+    my($self, @data) = @_;
+    my($data, $temp, $dir, $name, $path, $ref, $utf8);
 
     _fatal("unknown schema") unless $self->{type};
+    if (@data == 1) {
+	$data = $data[0];
+    } else {
+	$data = { @data };
+    }
     while (1) {
 	$temp = $self->{path} . "/" . TEMPORARY_DIRECTORY . "/" . _new_name();
 	last if _special_mkdir($temp, $self->{umask});
     }
-    foreach $name (keys(%data)) {
+    foreach $name (keys(%$data)) {
 	_fatal("unexpected data: %s", $name) unless $self->{type}{$name};
-	if ($self->{type}{$name} eq "binary") {
-	    _fatal("unexpected binary data in %s: %s", $name, $data{$name})
-		if ref($data{$name});
-	    eval {
-		_file_write("$temp/$name", 0, $self->{umask}, $data{$name});
-	    };
-	} elsif ($self->{type}{$name} eq "string") {
-	    _fatal("unexpected string data in %s: %s", $name, $data{$name})
-		if ref($data{$name});
-	    eval {
-		_file_write("$temp/$name", 1, $self->{umask}, $data{$name});
-	    };
+	if ($self->{type}{$name} =~ /^(binary|string)$/) {
+	    if ($self->{ref}{$name}) {
+		_fatal("unexpected %s data in %s: %s",
+		       $self->{type}{$name}, $name, $data->{$name})
+		    unless ref($data->{$name}) eq "SCALAR";
+		$ref = $data->{$name};
+	    } else {
+		_fatal("unexpected %s data in %s: %s",
+		       $self->{type}{$name}, $name, $data->{$name})
+		    if ref($data->{$name});
+		$ref = \$data->{$name};
+	    }
+	    $utf8 = $self->{type}{$name} eq "string";
 	} elsif ($self->{type}{$name} eq "table") {
-	    _fatal("unexpected table data in %s: %s", $name, $data{$name})
-		unless ref($data{$name}) eq "HASH";
-	    eval {
-		_file_write("$temp/$name", 1, $self->{umask}, _hash2string($data{$name}));
-	    };
+	    _fatal("unexpected %s data in %s: %s", $self->{type}{$name}, $name, $data->{$name})
+		unless ref($data->{$name}) eq "HASH";
+	    $ref = _hash2string($data->{$name});
+	    $utf8 = 1;
 	} else {
 	    _fatal("unexpected data type in %s: %s", $name, $self->{type}{$name});
 	}
+	eval {
+	    _file_write("$temp/$name", $utf8, $self->{umask}, $ref);
+	};
 	if ($@) {
 	    if ($@ =~ /^Wide character in /) {
-		_fatal("unexpected wide character in %s: %s", $name, $data{$name});
+		_fatal("unexpected wide character in %s: %s", $name, $data->{$name});
 	    } else {
 		die($@);
 	    }
@@ -790,18 +857,38 @@ sub add : method {
     }
     foreach $name (keys(%{ $self->{mandatory} })) {
 	_fatal("missing mandatory data: %s", $name)
-	    unless defined($data{$name});
+	    unless defined($data->{$name});
     }
     $dir = $self->_insertion_directory();
     while (1) {
 	$name = $dir . "/" . _new_name();
 	$path = $self->{path} . "/" . $name;
 	rename($temp, $path) and return($name);
-	_fatal("cannot rename(%s, %s): %s", $temp, $path, $!) unless $! == ENOTEMPTY;
+	_fatal("cannot rename(%s, %s): %s", $temp, $path, $!)
+	    unless $! == ENOTEMPTY or $! == EEXIST;
 	# RACE: the target directory was already present...
     }
 }
 
+#
+# return the list of volatile (i.e. temporary or obsolete) directories
+#
+
+sub _volatile : method {
+    my($self) = @_;
+    my(@list, $name);
+
+    foreach $name (_directory_contents($self->{path} . "/" . TEMPORARY_DIRECTORY, 1)) {
+	push(@list, TEMPORARY_DIRECTORY . "/" . $1)
+	    if $name =~ /^($_ElementRegexp)$/o; # untaint
+    }
+    foreach $name (_directory_contents($self->{path} . "/" . OBSOLETE_DIRECTORY, 1)) {
+	push(@list, OBSOLETE_DIRECTORY . "/" . $1)
+	    if $name =~ /^($_ElementRegexp)$/o; # untaint
+    }
+    return(@list);
+}
+
 #
 # purge the queue:
 #  - delete unused intermediate directories
@@ -816,8 +903,8 @@ sub purge : method {
     my(@list, $name, $path, $subdirs, $oldtime, $file, $fpath);
 
     # check options
-    $option{maxtemp} ||= 300;	# maximum time for a temporary element
-    $option{maxlock} ||= 600;	# maximum time for a locked element
+    $option{maxtemp} = 300 unless defined($option{maxtemp});
+    $option{maxlock} = 600 unless defined($option{maxlock});
     foreach $name (keys(%option)) {
 	_fatal("unexpected option: %s", $name)
 	    unless $name =~ /^(maxtemp|maxlock)$/;
@@ -840,42 +927,34 @@ sub purge : method {
 	    _special_rmdir($path);
 	}
     }
-    # get the list of temporary and obsolete directories
-    @list = ();
-    foreach $name (_directory_contents($self->{path} . "/" . TEMPORARY_DIRECTORY, 1)) {
-	push(@list, TEMPORARY_DIRECTORY . "/" . $1)
-	    if $name =~ /^($_ElementRegexp)$/o; # untaint
-    }
-    foreach $name (_directory_contents($self->{path} . "/" . OBSOLETE_DIRECTORY, 1)) {
-	push(@list, OBSOLETE_DIRECTORY . "/" . $1)
-	    if $name =~ /^($_ElementRegexp)$/o; # untaint
-    }
-    # remove the ones which are too old
-    $oldtime = time() - $option{maxtemp};
-    foreach $name (@list) {
-	$path = $self->{path} . "/" . $name;
-	next unless _older($path, $oldtime);
-	warn("* removing too old volatile element: $name\n");
-	foreach $file (_directory_contents($path, 1)) {
-	    next if $file eq LOCKED_DIRECTORY;
-	    $fpath = "$path/$file";
-	    unlink($fpath) and next;
-	    _fatal("cannot unlink(%s): %s", $fpath, $!) unless $! == ENOENT;
+    # remove the volatile directories which are too old
+    if ($option{maxtemp}) {
+	$oldtime = time() - $option{maxtemp};
+	foreach $name ($self->_volatile()) {
+	    $path = $self->{path} . "/" . $name;
+	    next unless _older($path, $oldtime);
+	    warn("* removing too old volatile element: $name\n");
+	    foreach $file (_directory_contents($path, 1)) {
+		next if $file eq LOCKED_DIRECTORY;
+		$fpath = "$path/$file";
+		unlink($fpath) and next;
+		_fatal("cannot unlink(%s): %s", $fpath, $!) unless $! == ENOENT;
+	    }
+	    _special_rmdir($path . "/" . LOCKED_DIRECTORY);
+	    _special_rmdir($path);
 	}
-	_special_rmdir($path . "/" . LOCKED_DIRECTORY);
-	_special_rmdir($path);
     }
     # iterate to find abandoned locked entries
-    $oldtime = time() - $option{maxlock};
-    $name = $self->first();
-    while ($name) {
-	$path = $self->{path} . "/" . $name;
-	next unless $self->_state($name) eq STATE_LOCKED;
-	next unless _older($path, $oldtime);
-	warn("* removing too old locked element: $name\n");
-	$self->unlock($name, 1);
-    } continue {
-	$name = $self->next();
+    if ($option{maxlock}) {
+	$oldtime = time() - $option{maxlock};
+	$name = $self->first();
+	while ($name) {
+	    next unless $self->_is_locked($name, $oldtime);
+	    warn("* removing too old locked element: $name\n");
+	    $self->unlock($name, 1);
+	} continue {
+	    $name = $self->next();
+	}
     }
 }
 
@@ -952,13 +1031,13 @@ using the same queue. Consider for instance:
 
 =over
 
-=item . Writer1: calls the add() method
+=item * Writer1: calls the add() method
 
-=item . Writer2: calls the add() method
+=item * Writer2: calls the add() method
 
-=item . Writer2: the add() method returns
+=item * Writer2: the add() method returns
 
-=item . Writer1: the add() method returns
+=item * Writer1: the add() method returns
 
 =back
 
@@ -1061,6 +1140,11 @@ By default, all pieces of data are mandatory. If you append a question
 mark to the type, this piece of data will be marked as optional. See
 the comments in the L</SYNOPSIS> section for more information.
 
+By default, string or binary data is used directly. If you append an
+asterisk to the type, the data that you add or get will be by
+reference. This can be useful to avoid string copies of large amounts
+of data.
+
 =head1 METHODS
 
 The following methods are available:
@@ -1100,8 +1184,9 @@ return an empty string if there is no next element
 
 =item add(DATA)
 
-add the given data (a hash) to the queue and return the corresponding
-element name; the schema must be known and the data must conform to it
+add the given data (a hash or hash reference) to the queue and return
+the corresponding element name; the schema must be known and the data
+must conform to it
 
 =item lock(ELEMENT[, PERMISSIVE])
 
@@ -1122,23 +1207,28 @@ remove the given element (which must be locked) from the queue
 =item get(ELEMENT)
 
 get the data from the given element (which must be locked) and return
-basically the same hash as what add() used; the schema must be known
+basically the same hash as what add() used (in list context, the hash
+is returned directly; in scalar context, the hash reference is
+returned instead); the schema must be known
 
 =item purge([OPTIONS])
 
 purge the queue by removing unused intermediate directories, removing
 too old temporary elements and unlocking too old locked elements (aka
-staled locks); OPTIONS can be:
+staled locks); note: this can take a long time on queues with many
+elements; OPTIONS can be:
 
 =over
 
 =item maxtemp
 
-maximum time for a temporary element (in seconds, default 300)
+maximum time for a temporary element (in seconds, default 300);
+if set to 0, temporary elements will not be removed
 
 =item maxlock
 
-maximum time for a locked element (in seconds, default 600)
+maximum time for a locked element (in seconds, default 600);
+if set to 0, locked elements will not be unlocked
 
 =back
 
@@ -1211,4 +1301,4 @@ enough to control who can access the queue.
 
 Lionel Cons L<http://cern.ch/lionel.cons>
 
-Copyright CERN 2010
+Copyright CERN 2010-2011
